require 'fileutils'
require 'open3'

silverdir="/silverware"
OBJDIR="/root/objects"
bindir="/usr/local/ARM_Compiler_5.06u6/bin"

src=["#{silverdir}/Silverware/src", "#{silverdir}/Utilities", "#{silverdir}/Libraries/STM32F0xx_StdPeriph_Driver/src"]
includes="-I #{silverdir}/Silverware/src -I #{silverdir}/Libraries/CMSIS/Device/ST/STM32F0xx/Include -I #{silverdir}/Libraries/CMSIS/Include -I #{silverdir}/Utilities -I #{silverdir}/Libraries/STM32F0xx_StdPeriph_Driver/inc"
startup="#{silverdir}/Libraries/CMSIS/Device/ST/STM32F0xx/Source/Templates/arm/startup_stm32f031.s"

cpu="--cpu Cortex-M0"
opt="-c #{cpu} -D__EVAL -D__MICROLIB -g -O2 --apcs=interwork --split_sections -D__UVISION_VERSION=\"524\" -DUSE_STDPERIPH_DRIVER -DSTM32F031 --fpmode=fast"
lst="#{cpu} --pd \"__EVAL SETA 1\" -g --apcs=interwork --pd \"__MICROLIB SETA 1\" --pd \"__UVISION_VERSION SETA 524\" --xref"
link="#{cpu} --library_type=microlib --ro-base 0x08000000 --entry 0x08000000 --rw-base 0x20000000 --entry Reset_Handler --first __Vectors --strict --info summarysizes"

ENV['ARM_TOOL_VARIANT']="mdk_lite"
ENV['ARMCC5_ASMOPT']="--diag_suppress=9931"
ENV['ARMCC5_CCOPT']="--diag_suppress=9931"
ENV['ARMCC5_LINKOPT']="--diag_suppress=9931"
ENV['CPU_TYPE']="STM32F030F4"
ENV['CPU_VENDOR']="STMicroelectronics"
ENV['CPU_CLOCK']="0x00B71B00"
ENV['UV2_TARGET']="BWhoop"

def run(command)
  stdin, stdout, stderr, wait_thr = Open3.popen3(command)
  error = stderr.read
  stdin.close
  stdout.close
  stderr.close
  if not wait_thr.value.success?
    $stderr.puts error
    `rm -rf #{OBJDIR}`
    exit 1
  end
end

FileUtils.mkdir OBJDIR
objlist = []

src.each do |srcdir|
  Dir.glob("#{srcdir}/*.c").each do |cfile|
    targetname = File.basename cfile, ".c"
    target = "#{OBJDIR}/#{targetname}.o"
    run "#{bindir}/armcc --c99 #{opt} #{includes} -o #{target} #{cfile}"
    objlist.push target
  end
  Dir.glob("#{srcdir}/*.cpp").each do |cppfile|
    targetname = File.basename cppfile, ".cpp"
    target = "#{OBJDIR}/#{targetname}.o"
    run "#{bindir}/armcc --cpp #{opt} #{includes} -o #{target} #{cppfile}"
    objlist.push target
  end
end

run "#{bindir}/armasm #{lst} -o #{OBJDIR}/startup_stm32f031.o #{startup}"
objlist.push "#{OBJDIR}/startup_stm32f031.o"

run "#{bindir}/armlink #{link} #{objlist.join ' '} -o #{OBJDIR}/silverware.axf"
run "#{bindir}/fromelf #{OBJDIR}/silverware.axf --i32combined --output /output/silverware.hex"

`rm -rf #{OBJDIR}`
